package com.mediaworx.b4f.opencms.contenttypes.widgets;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.opencms.widgets.I_CmsWidget;

/**
 * Custom widget for the selection of managed list entries in two steps (1st step: select the list name,
 * 2nd step: select the list entry).<br />
 * Works in concert with the JavaScript file
 * com.mediaworx.b4f.opencms.contenttypes/resources/listselector/ListSelectorWidget.js
 */
public class ListSelectorWidget extends B4fWidget {

	private static final Log LOG = LogFactory.getLog(ListSelectorWidget.class);

	/** JavaScript files to be included for the widget */
	private static final String[] JSPATHS = {
			"listselector/ListSelectorWidget.js"
	};

	/** CSS files to be included for the widget */
	private static final String[] CSSPATHS = {
			"listselector/ListSelectorWidget.css"
	};


	/** default constructor, necessary for registering the class */
	public ListSelectorWidget() {
		this("");
	}

	/**
	 * creates a new ListSelectorWidget widget with the provided configuration
	 *
	 * @param configuration configuration string (from the attribute "configuration" in the xsd file)
	 */
	public ListSelectorWidget(String configuration) {
		super(configuration);
	}

	/**
	 * returns the widget name and identifier that is also used in the client side javascript
	 * @return "ListSelectorWidget"
	 */
	@Override
	public String getWidgetName() {
		return "ListSelectorWidget";
	}

	/**
	 * returns the name of the JavaScript function that is called to initialize the widget
	 * @return "b4fListSelectorWidgetInitializationCall"
	 */
	@Override
	public String getInitCall() {
		if (LOG.isTraceEnabled()) {
			LOG.trace("getInitCall called");
		}
		// return the javascript function name that initializes the widget
		return "b4fListSelectorWidgetInitializationCall";
	}


	/**
	 * returns paths to css files (relative to {@link #RESOURCES_PATH}) that will be integrated in the xml content
	 * editor and the inline editor
	 * @return String array containing css paths
	 */
	@Override
	public String[] getCssPaths() {
		return CSSPATHS;
	}

	/**
	 * returns paths to JavaScript files (relative to {@link #RESOURCES_PATH}) that will be integrated in the xml
	 * content editor and the inline editor
	 * @return String array containing JavaScript paths
	 */
	@Override
	public String[] getJsPaths() {
		return JSPATHS;
	}

	/**
	 * Creates a new ListSelectorWidget instance
	 */
	@Override
	public I_CmsWidget newInstance() {
		return new ListSelectorWidget(getConfiguration());
	}

}
