/*
################################################################################

com.mediaworx.b4f.opencms.contenttypes/resources/listselector/ListSelectorWidget.js
mediaworx Berlin AG, Kai Widmann (widmann@mediaworx.com), 2013-03-22

FUNCTION:
	Integrates a two step selectbox widget into the XML Content Editor and the 
	Inline-Editor.
	Working in concert with the java class
	com.mediaworx.b4f.opencms.contenttypes.widgets.ListSelectorWidget

INPUT PARAMETERS:
	configuration - Configuration string from the java class (see method
	                getConfiguration)

################################################################################
 */

var listSelectorWidgetDebug = false;

var listSelectorWidgetConnectorUrl = "/b4fcms/system/modules/com.mediaworx.b4f.opencms.contenttypes/connectors/ManagedListConnector.jsp";
var listSelectorWidgetUriPrefix = "/.content/listentries/";
var listSelectorWidgetNameCounter = {};
var listSelectorWidgetListNames = null;
var listSelectorWidgetListEntries = {};
var listSelectorWidgetLastListName = null;

/**
 * does an AJAX request to retrieve the list entries in a specified list from the CMS
 * Works together with the JSP com.mediaworx.b4f.opencms.contenttypes/connectors/ManagedListConnector.jsp
 * @param listName  name of the list
 * @returns a parsed JSON response (key/value map)
 */
function getListSelectorWidgetListEntities(/*String*/listName) {
	var url = listSelectorWidgetConnectorUrl;
	if (listName != null) {
		url += "?listname="+listName;
	}
	var httpRequest = new XMLHttpRequest();
	httpRequest.open('GET', url, false);
	httpRequest.send(null);
	var response = httpRequest.responseText;
	return JSON.parse(response);
}

/**
 * does an AJAX request to retrieve the list names from the CMS
 * Works together with the JSP com.mediaworx.b4f.opencms.contenttypes/connectors/ManagedListConnector.jsp
 * @returns a parsed JSON response (key/value map)
 */
function getListSelectorWidgetListNames() {
	if (listSelectorWidgetListNames == null) {
		if (listSelectorWidgetDebug) {
			console.log("ListSelectorWidget - getting ListNames via JSON request");
		}
		listSelectorWidgetListNames = getListSelectorWidgetListEntities(null);
	}
	return listSelectorWidgetListNames;
}

/**
 * cached version of "getListSelectorWidgetListEntities", a list is retrieved only once and then stored in a map.
 * @param listName  name of the list
 * @returns a parsed JSON response (key/value map)
 */
function getListSelectorWidgetListEntries(/*String*/listName) {

	if (listSelectorWidgetListEntries[listName] == undefined) {
		listSelectorWidgetListNames[listName] = getListSelectorWidgetListEntities(listName);
	}
	return listSelectorWidgetListNames[listName];
}


/**
 * generic method to create the widgets for the XML content editor and the inline editor
 *
 * @param configuration Configuration string from the java class (see method getConfiguration)
 * @param element       if inline==true, then the DOM-Element from the preview, null otherwise
 * @param inline        true if called from the inline editor, false otherwise
 *
 * @return Widget object
 */
function createGenericB4fListSelectorWidget(/*String*/configuration, /*Element*/element, /*boolean*/inline) {
	if (listSelectorWidgetDebug) {
		console.log("createGenericB4fListSelectorWidget called. configuration: " + configuration);
	}

	// handle the configuration
	var b4fSelectorConfig = {};
	var keyValuePairs = configuration.split("&");
	for (var i = 0; i < keyValuePairs.length; i++) {
		var keyValuePair = keyValuePairs[i].split("=");
		b4fSelectorConfig[keyValuePair[0]] = keyValuePair[1];
	}

	// handle naming conflicts of repetitive elements
	if (listSelectorWidgetNameCounter[b4fSelectorConfig["name"]] >= 0) {
		listSelectorWidgetNameCounter[b4fSelectorConfig["name"]]++;
		b4fSelectorConfig["name"] = b4fSelectorConfig["name"] + "_" + listSelectorWidgetNameCounter[b4fSelectorConfig["name"]];
	}
	listSelectorWidgetNameCounter[b4fSelectorConfig["name"]] = 0;
	if (listSelectorWidgetDebug) {
		console.log("ListSelectorWidget - configured name: " + b4fSelectorConfig["name"]);
	}

	var nameSelectId = b4fSelectorConfig["name"] + "_nameselect";
	var entrySelectId = b4fSelectorConfig["name"] + "_entryselect";

	/*
	 * The widget object definition
	 */
	var widget = {
		_value: "",
		_element: element,
		_active: true,
		_listNameSelect: null,
		_listEntrySelect: null,

		/**
		 * Returns the widget root element.
		 */
		getElement: function () {
			if (listSelectorWidgetDebug) {
				console.log("ListSelectorWidget.getElement called for " + b4fSelectorConfig["name"]);
			}
			var listNames = getListSelectorWidgetListNames();

			// Define name select
			var nameSelect = document.createElement("select");
			nameSelect.id = nameSelectId;
			nameSelect.setAttribute("class", "listSelectorNameSelect");

			var option = document.createElement("option");
			option.setAttribute("value", "");
			option.innerHTML = "Liste auswählen";
			nameSelect.appendChild(option);

			for (var i = 0; i < listNames.length; i++) {
				option = document.createElement("option");
				option.setAttribute("value", listNames[i].uri);
				option.innerHTML = listNames[i].name;
				nameSelect.appendChild(option);
			}

			nameSelect.onchange = this.onListNameSelect;

			this._listNameSelect = nameSelect;


			// Define entry select
			var entrySelect = document.createElement("select");
			entrySelect.id = entrySelectId;
			entrySelect.setAttribute("class", "listSelectorEntrySelect");

			var option = document.createElement("option");
			option.setAttribute("value", "");
			option.innerHTML = "Listeneintrag auswählen";
			entrySelect.appendChild(option);
			entrySelect.disabled = true;

			entrySelect.onchange = this.onListEntrySelect;

			this._listEntrySelect = entrySelect;

			if (!inline) {
				this._element = document.createElement("div");
				this._element.setAttribute("class", "listSelector");
			}
			else {
				if (this._element.firstChild) {
					var uriComment = this._element.firstChild;
					var uri = uriComment.data;
					if (listSelectorWidgetDebug) {
						console.log("list entry uri: " + uri);
					}

					this.setValue(uri, false);
				}

				this._element.setAttribute("class", "listSelectorInline");
				this._element.innerHTML = "";
			}

			this._element.name = b4fSelectorConfig["name"];
			this._element.appendChild(nameSelect);
			this._element.appendChild(entrySelect);

			return this._element;
		},

		onListNameSelect: function () {
			var nameSelect = widget._listNameSelect;
			var entrySelect = widget._listEntrySelect;

			// remove all but the first option from the entry select
			while(entrySelect.options.length > 1) {
				entrySelect.remove(entrySelect.options.length - 1);
			}

			var selectedListName = nameSelect.options[nameSelect.selectedIndex].value;
			if (selectedListName.length > 0) {

				listSelectorWidgetLastListName = selectedListName;

				var listEntries = getListSelectorWidgetListEntries(selectedListName);

				for (var i = 0; i < listEntries.length; i++) {
					var option = document.createElement("option");
					option.setAttribute("value", listEntries[i].uri);
					option.innerHTML = listEntries[i].name;
					entrySelect.appendChild(option);
				}

				entrySelect.disabled = false;
			}
			else {
				if (listSelectorWidgetDebug) {
					console.log("clearing ListSelectWidget value - " + b4fSelectorConfig["name"]);
				}
				widget._value = "";
				if (typeof widget.onChangeCommand === "function") {
					widget.onChangeCommand();
				}
				entrySelect.disabled = true;
			}
		},

		onListEntrySelect: function () {
			var entrySelect = widget._listEntrySelect;
			var selectedValue = entrySelect.options[entrySelect.selectedIndex].value;

			if (listSelectorWidgetDebug) {
				console.log("setting ListSelectWidget value to "+selectedValue+" - " + b4fSelectorConfig["name"]);
			}
			widget._value = selectedValue;

			if (typeof widget.onChangeCommand === "function") {
				widget.onChangeCommand();
			}
		},

		/**
		 * Will be called once the widget element is attached to the DOM.
		 */
		onAttachWidget: function () {
			if (listSelectorWidgetDebug) {
				console.log("ListSelectorWidget.onAttachWidget called for " + b4fSelectorConfig["name"]);
			}
			if (this._value == null || this._value.length == 0) {
				if (listSelectorWidgetDebug) {
					console.log("There is no value, so let's find out the selected list name of the previous selectbox.");
				}
				var nameSelect = this._listNameSelect;
				try {
					var previousSelectContainer = nameSelect.parentNode.parentNode.parentNode.parentNode.previousSibling;
					var previousSelect = previousSelectContainer.firstChild.firstChild.firstChild.nextSibling.firstChild;
					if (listSelectorWidgetDebug) {
						console.log("previousSelect: " + previousSelect.id);
					}
					nameSelect.selectedIndex = previousSelect.selectedIndex;
					this.onListNameSelect();
				}
				catch (e) {
					if (listSelectorWidgetDebug) {
						console.log("ListSelectorWidget - Exception getting the previous select: " + e);
					}
				}
			}
		},

		/**
		 * Returns if the widget is active.
		 */
		isActive: function () {
			if (listSelectorWidgetDebug) {
				console.log("ListSelectorWidget.isActive called for " + b4fSelectorConfig["name"]);
			}
			return this._active;
		},

		/**
		 * Activates or deactivates the widget.
		 */
		setActive: function (/*boolean*/active) {
			if (listSelectorWidgetDebug) {
				console.log("ListSelectorWidget.setActive called for " + b4fSelectorConfig["name"]);
			}
			this._active = active;
		},

		/**
		 * Returns the widget value.
		 */
		getValue: function () {
			return this._value
		},

		/**
		 * Sets the widget value and fires the change event if required.
		 */
		setValue: function (/*String*/value, /*boolean*/fireEvent) {
			if (listSelectorWidgetDebug) {
				console.log("ListSelectorWidget.setValue called for " + b4fSelectorConfig["name"] + ". value: " + value);
			}
			// select the list name
			var nameSelect = this._listNameSelect;

			var listName;

			if (value != null && value.length > 0) {
				listName = value.substr(listSelectorWidgetUriPrefix.length);
				listName = listName.substr(0, listName.indexOf("/"));
			}
			else {
				listName = listSelectorWidgetLastListName;
			}

			if (listName != null && listName.length > 0) {
				for (var i = 0; i < nameSelect.options.length; i++) {
					var option = nameSelect.options[i];
					if (option.value == listName) {
						nameSelect.selectedIndex = i;
						this.onListNameSelect();

						if (value.length > 0) {
							// select the list entry
							var entrySelect = this._listEntrySelect;
							for (var j = 0; j < entrySelect.length; j++) {
								option = entrySelect.options[j];
								if (option.value.indexOf(value) > -1) {
									entrySelect.selectedIndex = j;
									break;
								}
							}
						}
						break;
					}
				}
			}

			this._value = value;
			if (fireEvent && typeof this.onChangeCommand === "function") {
				this.onChangeCommand();
			}
		},

		/**
		 * Delegates the value change event to the editor.
		 * This function will be attached to the widget by the editor.
		 *
		 * It is required to call this function everytime the widget value changes.
		 * Changes that are propagated through this function won't be saved.
		 */
		onChangeCommand: null,

		/**
		 * Delegates the focus event to the editor.
		 * This function will be attached to the widget by the editor.
		 *
		 * It is required to call this function on widget focus, so the editor
		 * highlighting can be updated.
		 */
		onFocusCommand: null
	};
	return widget;
}

/**
 * Initialisierungs-Methode, erzeugt ein neues ListSelector-Widget
 */
function b4fListSelectorWidgetInitializationCall() {

	registerWidgetFactory({

		/**
		 * The widget name and identifier.
		 */
		widgetName: "ListSelectorWidget",
		/**
		 * Creates a new widget instance according to the configuration.
		 */
		createNativeWidget: function (/*String*/configuration) {
			return createGenericB4fListSelectorWidget(configuration, null, false);
		},


		/**
		 * Creates a new inline editing widget instance according to the configuration.
		 */
		createNativeWrapedElement: function (/*String*/configuration, /*Element*/ element) {
			return createGenericB4fListSelectorWidget(configuration, element, true);
		}
	});
}
